# Visual UV
# Copyright (C) 2022 Samuel Bílek

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from . visualuv_ui import *
from . visualuv_ops import *

from bpy.types import PropertyGroup
from bpy.props import IntProperty, BoolProperty, StringProperty, CollectionProperty, FloatVectorProperty, FloatProperty
import bpy


bl_info = {
    "name": "Visual UV",
    "author": "Samuel Bílek",
    "description": "Adds UV visualisation overlays for meshes. Useful for visualising UV stretching, individual UV islands and flipped UV normals.",
    "blender": (3, 4, 0),
    "version": (0, 0, 1),
    'location': 'VIEW_3D > Tools > UV Visualisation',
    "warning": "",
    "category": "Object"
}


classes = (
    VISUALUV_PT_uv_tool_menu3d,
    VISUALUV_PT_uv_tool_menu2d,
    VISUALUV_OT_update,
    VISUALUV_OT_toggle_texture,
    VISUALUV_OT_toggle_stretching,
    VISUALUV_OT_toggle_normals,
    VISUALUV_OT_toggle_overlapped,
    VISUALUV_OT_toggle_islands,
    VISUALUV_OT_overlay,
)

addon_keymaps = []




def register():
    for c in classes:
        bpy.utils.register_class(c)

    bpy.types.WindowManager.uv_select_overlap = BoolProperty(default=False)
    
    bpy.types.Object.rv_enabled = BoolProperty()
    bpy.types.Object.rv_backface_culling = BoolProperty()
    bpy.types.Object.rv_use_x_mirror = BoolProperty()
    bpy.types.Object.uv_show_wire = BoolProperty(default=False)
    bpy.types.Object.rv_show_poles = BoolProperty()

    bpy.types.Object.rv_index = IntProperty()
    bpy.types.Object.rv_group_idx_counter = IntProperty(default=1)

    bpy.types.Object.rv_groups_alpha = FloatProperty(default=0.75, max=1.0, min=0.0)

    bpy.types.Object.uv_recalculate = BoolProperty()
    bpy.types.Object.uv_overlap_recalculate = BoolProperty()
    bpy.types.Object.uv_realtime = BoolProperty()
    bpy.types.Object.uv_show_3D = BoolProperty(default=True)
    bpy.types.Object.uv_show_2D = BoolProperty(default=True)
    bpy.types.Object.uv_enable_position_change = BoolProperty()
    bpy.types.Object.uv_enable_color_change = BoolProperty()
    bpy.types.Object.uv_enable_explosion_view = BoolProperty()
    bpy.types.Object.uv_checker_texture = BoolProperty(default=False)
    bpy.types.Object.uv_fill_texture = BoolProperty(default=True)
    bpy.types.Object.uv_image = bpy.props.PointerProperty(name='Image', type=bpy.types.Image)

    bpy.types.Object.uv_texture_scale = FloatProperty(default=1.0, soft_max=2.0, min=0.01)
    bpy.types.Object.uv_texture_alpha = FloatProperty(default=1.0, max=1.0, min=0.00)
    bpy.types.Object.uv_max_difference = FloatProperty(default=0.25, min=0.01, soft_max=1.0)
    bpy.types.Object.uv_max_division = FloatProperty(default=2.0, min=1.01, soft_max=5.0)

    bpy.types.Object.uv_hue_shift = FloatProperty(default=0.0, max=1.0, min=0.00)
    bpy.types.Object.uv_saturation = FloatProperty(default=1.0, max=1.0, min=0.00)
    bpy.types.Object.uv_value = FloatProperty(default=1.0, max=1.0, min=0.00)

    bpy.types.Object.uv_explosion_offset = FloatProperty(default=0.0, soft_max=3.00, min=0.00)
    bpy.types.Object.uv_location_offset = FloatVectorProperty(subtype='XYZ')
    bpy.types.Object.uv_operation = EnumProperty(items=[('NONE', "None", "", 1), ('UV_STRETCHING', "Stretching", "", 2), ('UV_ISLANDS', "Islands", "", 3), ('UV_NORMALS', "Normals", "", 4), ('UV_OVERLAP', "Overlap", "", 5)], default='NONE')
    bpy.types.Object.uv_stretch_type = EnumProperty(items=[('ANGLES', "Angles", "", 1), ('EDGE_LENGTH', "Edge length", "", 2)], default='ANGLES')

    bpy.types.Object.rv_poles_size = FloatProperty(default=1.0, max=2.0, min=0.0)

    bpy.types.Object.rv_poles_color = FloatVectorProperty(name="Poles Color", subtype='COLOR', default=[1.0, 1.0, 1.0], min=0.0, max=1.0)

    #wm = bpy.context.window_manager
    #km = wm.keyconfigs.addon.keymaps.new(name='3D View', space_type='VIEW_3D')
    #kmi = km.keymap_items.new('wm.call_menu_pie', 'F', 'PRESS', ctrl=False, shift=True, alt=False)
    #kmi.properties.name = "RETOPOVIEW_MT_rv_pie_menu"

    #addon_keymaps.append((km, kmi))


def unregister():
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()

    del bpy.types.Object.rv_poles_color

    del bpy.types.Object.rv_poles_size

    del bpy.types.Object.uv_stretch_type
    del bpy.types.Object.uv_operation
    del bpy.types.Object.uv_location_offset
    del bpy.types.Object.uv_explosion_offset
    del bpy.types.Object.uv_value
    del bpy.types.Object.uv_saturation
    del bpy.types.Object.uv_hue_shift
    del bpy.types.Object.uv_max_division
    del bpy.types.Object.uv_max_difference
    del bpy.types.Object.uv_texture_alpha
    del bpy.types.Object.uv_texture_scale
    del bpy.types.Object.uv_image
    del bpy.types.Object.uv_fill_texture
    del bpy.types.Object.uv_checker_texture
    del bpy.types.Object.uv_enable_explosion_view
    del bpy.types.Object.uv_enable_color_change
    del bpy.types.Object.uv_enable_position_change
    del bpy.types.Object.uv_show_2D
    del bpy.types.Object.uv_show_3D
    del bpy.types.Object.uv_realtime
    del bpy.types.Object.uv_overlap_recalculate
    del bpy.types.Object.uv_recalculate
    del bpy.types.Object.rv_groups_alpha


    del bpy.types.Object.rv_group_idx_counter
    del bpy.types.Object.rv_index

    del bpy.types.Object.rv_show_poles
    del bpy.types.Object.uv_show_wire
    del bpy.types.Object.rv_use_x_mirror
    del bpy.types.Object.rv_backface_culling
    del bpy.types.Object.rv_enabled

    del bpy.types.WindowManager.uv_select_overlap

    for c in classes:
        bpy.utils.unregister_class(c)


if __name__ == "__main__":
    register()
